using System;

namespace Sharp3D
{
	public class BumpMapVertexShader : IVertexShader
	{
		private Matrix matrixLocal2Canonical;
		private Point3D posLight, posEye;	// local coordinate system

		public BumpMapVertexShader(Matrix matrixLocal2Canonical, Point3D posLight, Point3D posEye)
		{
			this.matrixLocal2Canonical = matrixLocal2Canonical;
			this.posLight = posLight;
			this.posEye = posEye;
		}

		public Vertex calculate(Vertex input)
		{
			// transform position to clip space
			Vertex output = new Vertex(matrixLocal2Canonical * input.Position);

			// emit texture coordinates
			output.TextureCoordinate0 = input.TextureCoordinate0;

			// view, light and half vector

			Vector3D lightVec = (posLight - input.Position).GetNormalized();
			Vector3D viewVec = (posEye - input.Position).GetNormalized();
			Vector3D halfVec = (lightVec + viewVec).GetNormalized();

			// transform to tangent space;
			// w is same direction as normal

			// binormal
			Vector3D binormal = input.Normal % input.Tangent;

			output.TextureCoordinate1.X = lightVec * input.Tangent;
			output.TextureCoordinate1.Y = lightVec * binormal;
			output.TextureCoordinate1.Z = lightVec * input.Normal;

			output.TextureCoordinate2.X = halfVec * input.Tangent;
			output.TextureCoordinate2.Y = halfVec * binormal;
			output.TextureCoordinate2.Z = halfVec * input.Normal;

			return output;
		}
	}
}
