using System;

namespace Sharp3D
{
	public class CameraException : System.ApplicationException
	{
		public CameraException(string message) : base(message) 
		{
		}
	}

	public class Camera
	{
		private Point3D position, center;
		private Vector3D up, dir;

		private Rectangle2D viewRect;
		private double nearPlaneDistance, farPlaneDistance;

		public Camera(Point3D position, Point3D center, Vector3D up, Rectangle2D viewRect,
			double nearPlaneDistance, double farPlaneDistance)
		{
			this.position = position;
			this.center = center;

			this.dir = (center - position).GetNormalized();
			this.up = ((dir % up) % dir).GetNormalized();

			this.viewRect = viewRect;
			this.nearPlaneDistance = nearPlaneDistance;
			this.farPlaneDistance = farPlaneDistance;
		}

		public Point3D Position
		{
			get
			{
				return position;
			}

			set
			{
				position = value;

				dir = (center - position).GetNormalized();
				up = ((dir % up) % dir).GetNormalized();
			}
		}

		public Point3D Center
		{
			get
			{
				return center;
			}

			set
			{
				center = value;

				dir = (center - position).GetNormalized();
				up = ((dir % up) % dir).GetNormalized();
			}
		}

		public Vector3D Up
		{
			get
			{
				return up;
			}

			set
			{
				up = ((dir % value) % dir).GetNormalized();
			}
		}

		public Vector3D Direction
		{
			get
			{
				return dir;
			}
		}

		public Rectangle2D ViewRect
		{
			get
			{
				return viewRect;
			}

			set
			{
				viewRect = value;
			}
		}

		public double NearPlaneDistance
		{
			get
			{
				return nearPlaneDistance;
			}

			set
			{
				nearPlaneDistance = value;
			}
		}
		
		public double FarPlaneDistance
		{
			get
			{
				return farPlaneDistance;
			}

			set
			{
				farPlaneDistance = value;
			}
		}

		public void SetViewingVolume(Rectangle2D viewRect, double nearPlaneDistance, double farPlaneDistance)
		{
			this.viewRect = viewRect;
			this.nearPlaneDistance = nearPlaneDistance;
			this.farPlaneDistance = farPlaneDistance;
		}

		public Matrix World2Camera
		{
			get
			{
				Vector3D r = dir % up;
				Vector3D c = new Vector3D(position.X, position.Y, position.Z);

				return new Matrix(
					r.X, r.Y, r.Z, -r*c,
					-up.X, -up.Y, -up.Z, up*c,
					dir.X, dir.Y, dir.Z, -dir*c,
					0.0, 0.0, 0.0, 1.0, true);
			}
		}

		public Matrix Camera2Canonical
		{
			get
			{
				return new Matrix(
					2.0*nearPlaneDistance/(farPlaneDistance*(viewRect.Right - viewRect.Left)), 0.0, (viewRect.Left + viewRect.Right)/(farPlaneDistance*(viewRect.Left - viewRect.Right)), 0.0,
					0.0, 2.0*nearPlaneDistance/(farPlaneDistance*(viewRect.Bottom - viewRect.Top)), (viewRect.Top + viewRect.Bottom)/(farPlaneDistance*(viewRect.Top - viewRect.Bottom)), 0.0,
					0.0, 0.0, 1.0/farPlaneDistance, 0.0,
					0.0, 0.0, 0.0, 1.0);
			}
		}

		public Matrix Canonical2Screen
		{
			get
			{
				return new Matrix(
					1.0, 0.0, 0.0, 0.0,
					0.0, 1.0, 0.0, 0.0,
					0.0, 0.0, farPlaneDistance/(farPlaneDistance - nearPlaneDistance), nearPlaneDistance/(nearPlaneDistance - farPlaneDistance),
					0.0, 0.0, 1.0, 0.0);
			}
		}
	}
}
