using System;

namespace Sharp3D
{
	public class Clipper3D
	{
		private double near;
		private double far;

		private bool IsInside(Point3D point, Point3D basePoint, Vector3D normal)
		{
			return (basePoint - point) * normal > 0.0;
		}

		private double Intersect(Point3D startPoint, Point3D endPoint, Point3D basePoint, Vector3D normal)
		{
			return ((basePoint - startPoint) * normal) / ((endPoint - startPoint) * normal);
		}

		private Polygon3D ClipPlane(Polygon3D polygon, Point3D basePoint, Vector3D normal)
		{
			Polygon3D result = new Polygon3D();

			for (int i = 0; i < polygon.Size; i++)
			{
				Point3D p = polygon[i].Position;
				Point3D q = polygon[(i+1)%polygon.Size].Position;

				if (IsInside(p, basePoint, normal))
				{
					if (IsInside(q, basePoint, normal)) 
					{
						result.Add(polygon[(i+1)%polygon.Size]);
					}
					else 
					{
						double lambda = Intersect(p, q, basePoint, normal);
						result.Add(
							new Vertex(p + lambda*(q - p),
							(polygon[i].Normal + lambda*(polygon[(i+1)%polygon.Size].Normal - polygon[i].Normal)).GetNormalized(),
							(polygon[i].Tangent + lambda*(polygon[(i+1)%polygon.Size].Tangent - polygon[i].Tangent)).GetNormalized(),
							polygon[i].DiffuseWeight + lambda*(polygon[(i+1)%polygon.Size].DiffuseWeight - polygon[i].DiffuseWeight),
							polygon[i].SpecularWeight + lambda*(polygon[(i+1)%polygon.Size].SpecularWeight - polygon[i].SpecularWeight),
							polygon[i].TextureCoordinate0 + lambda*(polygon[(i+1)%polygon.Size].TextureCoordinate0 - polygon[i].TextureCoordinate0),
							polygon[i].TextureCoordinate1 + lambda*(polygon[(i+1)%polygon.Size].TextureCoordinate1 - polygon[i].TextureCoordinate1),
							polygon[i].TextureCoordinate2 + lambda*(polygon[(i+1)%polygon.Size].TextureCoordinate2 - polygon[i].TextureCoordinate2)));
					}
				}
				else
				{
					if (IsInside(q, basePoint, normal)) 
					{
						double lambda = Intersect(p, q, basePoint, normal);
						result.Add(
							new Vertex(p + lambda*(q - p),
							(polygon[i].Normal + lambda*(polygon[(i+1)%polygon.Size].Normal - polygon[i].Normal)).GetNormalized(),
							(polygon[i].Tangent + lambda*(polygon[(i+1)%polygon.Size].Tangent - polygon[i].Tangent)).GetNormalized(),
							polygon[i].DiffuseWeight + lambda*(polygon[(i+1)%polygon.Size].DiffuseWeight - polygon[i].DiffuseWeight),
							polygon[i].SpecularWeight + lambda*(polygon[(i+1)%polygon.Size].SpecularWeight - polygon[i].SpecularWeight),
							polygon[i].TextureCoordinate0 + lambda*(polygon[(i+1)%polygon.Size].TextureCoordinate0 - polygon[i].TextureCoordinate0),
							polygon[i].TextureCoordinate1 + lambda*(polygon[(i+1)%polygon.Size].TextureCoordinate1 - polygon[i].TextureCoordinate1),
							polygon[i].TextureCoordinate2 + lambda*(polygon[(i+1)%polygon.Size].TextureCoordinate2 - polygon[i].TextureCoordinate2)));

						result.Add(polygon[(i+1)%polygon.Size]);
					}
				}
			}

			return result;
		}

		public Clipper3D(double near, double far)
		{
			this.near = near;
			this.far = far;
		}

		public bool Clip(Polygon3D input, out Polygon3D output)
		{
			output = ClipPlane(input,  new Point3D(0.0, 0.0, 1.0), new Vector3D(0.0, 0.0, 1.0));
			output = ClipPlane(output, new Point3D(0.0, 0.0, near/far), new Vector3D(0.0, 0.0, -1.0));
			output = ClipPlane(output, new Point3D(0.0, 0.0, 0.0), new Vector3D(0.0, -0.5*Math.Sqrt(2.0), -0.5*Math.Sqrt(2.0)));
			output = ClipPlane(output, new Point3D(0.0, 0.0, 0.0), new Vector3D(0.0,  0.5*Math.Sqrt(2.0), -0.5*Math.Sqrt(2.0)));
			output = ClipPlane(output, new Point3D(0.0, 0.0, 0.0), new Vector3D(-0.5*Math.Sqrt(2.0), 0.0, -0.5*Math.Sqrt(2.0)));
			output = ClipPlane(output, new Point3D(0.0, 0.0, 0.0), new Vector3D(0.5*Math.Sqrt(2.0), 0.0, -0.5*Math.Sqrt(2.0)));

			return output.Size != 0;
		}
	}
}
