using System;

namespace Sharp3D
{
	public class ClipperRect2D
	{
		const int leftPattern = 0x01;
		const int rightPattern = 0x02;
		const int bottomPattern = 0x04;
		const int topPattern = 0x08;

		public Rectangle2D ClipRect;

		private int bitVal(Point2D point)
		{
			return
				(point.X < ClipRect.Left ? leftPattern : (point.X > ClipRect.Right ? rightPattern : 0)) |
				(point.Y > ClipRect.Bottom ? bottomPattern : (point.Y < ClipRect.Top ? topPattern : 0));
		}

		public ClipperRect2D(Rectangle2D rect)
		{
			ClipRect = rect;
		}

		public bool Clip(Point2D point)
		{
			return bitVal(point) == 0;
		}

		public bool Clip(Line2D line, ref Line2D clipped)
		{
			clipped = line;

			int codeP = bitVal(clipped.StartPoint);
			int codeQ = bitVal(clipped.EndPoint);

			while (true)
			{
				if ((codeP | codeQ) == 0)
				{
					// trivial accept
					return true;
				}

				if ((codeP & codeQ) != 0)
				{
					// trivial reject
					return false;
				}

				if (codeP != 0)
				{
					if ((codeP & leftPattern) != 0)
					{
						double scale = (ClipRect.Left - clipped.StartPoint.X) / (clipped.EndPoint.X - clipped.StartPoint.X);
						clipped.StartPoint.X = ClipRect.Left;
						clipped.StartPoint.Y += scale * (clipped.EndPoint.Y - clipped.StartPoint.Y);

						codeP = bitVal(clipped.StartPoint);
					}
					else if ((codeP & rightPattern) != 0)
					{
						double scale = (ClipRect.Right - clipped.StartPoint.X) / (clipped.EndPoint.X - clipped.StartPoint.X);
						clipped.StartPoint.X = ClipRect.Right;
						clipped.StartPoint.Y += scale * (clipped.EndPoint.Y - clipped.StartPoint.Y);

						codeP = bitVal(clipped.StartPoint);
					}
					else if ((codeP & bottomPattern) != 0)
					{
						double scale = (ClipRect.Bottom - clipped.StartPoint.Y) / (clipped.EndPoint.Y - clipped.StartPoint.Y);
						clipped.StartPoint.Y = ClipRect.Bottom;
						clipped.StartPoint.X += scale * (clipped.EndPoint.X - clipped.StartPoint.X);

						codeP = bitVal(clipped.StartPoint);
					}
					else
					{
						double scale = (ClipRect.Top - clipped.StartPoint.Y) / (clipped.EndPoint.Y - clipped.StartPoint.Y);
						clipped.StartPoint.Y = ClipRect.Top;
						clipped.StartPoint.X += scale * (clipped.EndPoint.X - clipped.StartPoint.X);

						codeP = bitVal(clipped.StartPoint);
					}
				}
				else	// codeQ != 0
				{
					if ((codeQ & leftPattern) != 0)
					{
						double scale = (ClipRect.Left - clipped.EndPoint.X) / (clipped.StartPoint.X - clipped.EndPoint.X);
						clipped.EndPoint.X = ClipRect.Left;
						clipped.EndPoint.Y += scale * (clipped.StartPoint.Y - clipped.EndPoint.Y);

						codeQ = bitVal(clipped.EndPoint);
					}
					else if ((codeQ & rightPattern) != 0)
					{
						double scale = (ClipRect.Right - clipped.EndPoint.X) / (clipped.StartPoint.X - clipped.EndPoint.X);
						clipped.EndPoint.X = ClipRect.Right;
						clipped.EndPoint.Y += scale * (clipped.StartPoint.Y - clipped.EndPoint.Y);

						codeQ = bitVal(clipped.EndPoint);
					}
					else if ((codeQ & bottomPattern) != 0)
					{
						double scale = (ClipRect.Bottom - clipped.EndPoint.Y) / (clipped.StartPoint.Y - clipped.EndPoint.Y);
						clipped.EndPoint.Y = ClipRect.Bottom;
						clipped.EndPoint.X += scale * (clipped.StartPoint.X - clipped.EndPoint.X);

						codeQ = bitVal(clipped.EndPoint);
					}
					else
					{
						double scale = (ClipRect.Top - clipped.EndPoint.Y) / (clipped.StartPoint.Y - clipped.EndPoint.Y);
						clipped.EndPoint.Y = ClipRect.Top;
						clipped.EndPoint.X += scale * (clipped.StartPoint.X - clipped.EndPoint.X);

						codeQ = bitVal(clipped.EndPoint);
					}
				}
			}
		}

		private Polygon2D clipEdgeLeft(Polygon2D polygon)
		{
			Polygon2D result = new Polygon2D();

			for (int i = 0; i < polygon.Size; i++)
			{
				Point2D p = polygon[i];
				Point2D q = polygon[(i+1)%polygon.Size];

				if (p.X >= ClipRect.Left)
				{
					if (q.X >= ClipRect.Left)
					{
						result.Add(q);
					}
					else
					{
						result.Add(new Point2D(ClipRect.Left, p.Y + (ClipRect.Left - p.X)/(q.X - p.X) * (q.Y - p.Y)));
					}
				}
				else
				{
					if (q.X >= ClipRect.Left)
					{
						result.Add(new Point2D(ClipRect.Left, p.Y + (ClipRect.Left - p.X)/(q.X - p.X) * (q.Y - p.Y)));
						result.Add(q);
					}
				}
			}

			return result;
		}

		private Polygon2D clipEdgeTop(Polygon2D polygon)
		{
			Polygon2D result = new Polygon2D();

			for (int i = 0; i < polygon.Size; i++)
			{
				Point2D p = polygon[i];
				Point2D q = polygon[(i+1)%polygon.Size];

				if (p.Y >= ClipRect.Top)
				{
					if (q.Y >= ClipRect.Top) 
					{
						result.Add(q);
					}
					else 
					{
						result.Add(new Point2D(p.X + (ClipRect.Top - p.Y)/(q.Y - p.Y) * (q.X - p.X), ClipRect.Top));
					}
				}
				else
				{
					if (q.Y >= ClipRect.Top)
					{
						result.Add(new Point2D(p.X + (ClipRect.Top - p.Y)/(q.Y - p.Y) * (q.X - p.X), ClipRect.Top));
						result.Add(q);
					}
				}
			}

			return result;
		}

		private Polygon2D clipEdgeRight(Polygon2D polygon)
		{
			Polygon2D result = new Polygon2D();

			for (int i = 0; i < polygon.Size; i++)
			{
				Point2D p = polygon[i];
				Point2D q = polygon[(i+1)%polygon.Size];

				if (p.X <= ClipRect.Right)
				{
					if (q.X <= ClipRect.Right) 
					{
						result.Add(q);
					}
					else 
					{
						result.Add(new Point2D(ClipRect.Right, p.Y + (ClipRect.Right - p.X)/(q.X - p.X) * (q.Y - p.Y)));
					}
				}
				else
				{
					if (q.X <= ClipRect.Right)
					{
						result.Add(new Point2D(ClipRect.Right, p.Y + (ClipRect.Right - p.X)/(q.X - p.X) * (q.Y - p.Y)));
						result.Add(q);
					}
				}
			}

			return result;
		}

		private Polygon2D clipEdgeBottom(Polygon2D polygon)
		{
			Polygon2D result = new Polygon2D();

			for (int i = 0; i < polygon.Size; i++)
			{
				Point2D p = polygon[i];
				Point2D q = polygon[(i+1)%polygon.Size];

				if (p.Y <= ClipRect.Bottom)
				{
					if (q.Y <= ClipRect.Bottom)
					{
						result.Add(q);
					}
					else
					{
						result.Add(new Point2D(p.X + (ClipRect.Bottom - p.Y)/(q.Y - p.Y) * (q.X - p.X), ClipRect.Bottom));
					}
				}
				else
				{
					if (q.Y <= ClipRect.Bottom)
					{
						result.Add(new Point2D(p.X + (ClipRect.Bottom - p.Y)/(q.Y - p.Y) * (q.X - p.X), ClipRect.Bottom));
						result.Add(q);
					}
				}
			}

			return result;
		}

		public bool Clip(Polygon2D polygon, ref Polygon2D clipped)
		{
			clipped = clipEdgeLeft(polygon);
			clipped = clipEdgeTop(clipped);
			clipped = clipEdgeRight(clipped);
			clipped = clipEdgeBottom(clipped);

			return clipped.Size != 0;
		}
	}
}
