using System;

namespace Sharp3D
{
	public class Color
	{
		int r, g, b, a;

		public Color(int r, int g, int b)
		{
			this.r = r;
			this.g = g;
			this.b = b;
			this.a = 255;
		}

		public Color(int r, int g, int b, int a)
		{
			this.r = r;
			this.g = g;
			this.b = b;
			this.a = a;
		}

		public Color(short value16)
		{
			r = (value16 & 0xf800) >> 8;
			g = (value16 & 0x7e0) >> 3;
			b = (value16 & 0x1f) << 3;
			a = 255;
		}

		public int Red
		{
			get
			{
				return r;
			}
			set
			{
				r = value;
			}
		}

		public int Green
		{
			get
			{
				return g;
			}
			set
			{
				g = value;
			}
		}

		public int Blue
		{
			get
			{
				return b;
			}
			set
			{
				b = value;
			}
		}

		public int Alpha
		{
			get
			{
				return a;
			}
			set
			{
				a = value;
			}
		}

		public int U16
		{
			get
			{
				return (short)((g << 8) | b);
			}
		}

		public int V16
		{
			get
			{
				return (short)((a << 8) | r);
			}
		}

		public int U8
		{
			get
			{
				return (sbyte)b;
			}
		}

		public int V8
		{
			get
			{
				return (sbyte)g;
			}
		}

		public int W8
		{
			get
			{
				return (sbyte)r;
			}
		}

		public int Q8
		{
			get
			{
				return (sbyte)a;
			}
		}

		public short Value16
		{
			get
			{
				// use only the most significant bits of each color channel

				return (short) (((r >> 3) << 11) | ((g >> 2) << 5) | (b >> 3));
			}
			set
			{
				r = (value & 0xf800) >> 8;
				g = (value & 0x7e0) >> 3;
				b = (value & 0x1f) << 3;
			}
		}

		public double RedScaled
		{
			get
			{
				return (double)r / 255.0;
			}
		}

		public double GreenScaled
		{
			get
			{
				return (double)g / 255.0;
			}
		}

		public double BlueScaled
		{
			get
			{
				return (double)b / 255.0;
			}
		}

		public double AlphaScaled
		{
			get
			{
				return (double)a / 255.0;
			}
		}

		public double U16Scaled
		{
			get
			{
				return U16 / 32768.0;
			}
		}

		public double V16Scaled
		{
			get
			{
				return V16 / 32768.0;
			}
		}

		public double U8Scaled
		{
			get
			{
				return U8 / 128.0;
			}
		}

		public double V8Scaled
		{
			get
			{
				return V8 / 128.0;
			}
		}

		public double W8Scaled
		{
			get
			{
				return W8 / 128.0;
			}
		}

		public double Q8Scaled
		{
			get
			{
				return Q8 / 128.0;
			}
		}

		public static Color operator+ (Color colorL, Color colorR)
		{
			Color result = new Color(
				colorL.r + colorR.r,
				colorL.g + colorR.g,
				colorL.b + colorR.b,
				(colorL.a + colorR.a)/2);

			if (result.r > 255)
				result.r = 255;

			if (result.g > 255)
				result.g = 255;

			if (result.b > 255)
				result.b = 255;

			return result;
		}

		public static Color operator* (double s, Color color)
		{
			if (s < 0.0)
				return new Color(0, 0, 0, color.a);
			else
				return new Color(
					(int)(s * color.r),
					(int)(s * color.g),
					(int)(s * color.b),
					color.a);
		}

		public static Color operator* (Color colorL, Color colorR)
		{
			return new Color(
				(colorL.r * colorR.r) / 255,
				(colorL.g * colorR.g) / 255,
				(colorL.b * colorR.b) / 255,
				(colorL.a * colorR.a) / 255);
		}
	}
}
