using System;

namespace Sharp3D
{
	public class DirectionalLight : ILight
	{
		private Color color;
		private Vector3D direction;

		public DirectionalLight(Color color, Vector3D direction)
		{
			this.color = color;
			this.direction = direction.GetNormalized();
		}

		public double CalcDiffuseWeight(Point3D position, Vector3D normal, Material material)
		{
			double sc = - direction * normal;

			if (sc < 0.0)
			{
				return 0.0;
			}
			else
			{
				return sc;
			}
		}

		public double CalcSpecularWeight(Point3D position, Vector3D normal, Point3D cameraPos, Material material)
		{
			Vector3D reflect = 2.0*(direction*normal)*normal - direction;
			Vector3D view = (cameraPos - position).GetNormalized();

			double sc = view * reflect;

			if (sc < 0.0)
			{
				return 0.0;
			}
			else
			{
				return Math.Pow(sc, material.SpecularExponent);
			}
		}

		public ILight GetTransformed(Matrix m)
		{
			return new DirectionalLight(color, m * direction);
		}

		public Color Color
		{
			get
			{
				return color;
			}
		}
	}
}
