using System;

namespace Sharp3D
{
	public class FunctionMeshFactory
	{
		public delegate double Evaluator(double x, double y);

		private Evaluator eval;

		private double xmin, xmax;
        private double ymin, ymax;
		private int xSteps, ySteps;

		public FunctionMeshFactory(Evaluator eval, double xmin, double xmax,
			double ymin, double ymax, int xSteps, int ySteps)
		{
			this.eval = eval;
			this.xmin = xmin;
			this.xmax = xmax;
			this.ymin = ymin;
			this.ymax = ymax;
			this.xSteps = xSteps;
			this.ySteps = ySteps;
		}

		public Mesh3D CreateMesh()
		{
			Vertex[,] grid = new Vertex[xSteps, ySteps];

			for (int i = 0; i < xSteps; i++)
			{
				for (int j = 0; j < ySteps; j++)
				{
					grid[i, j] = new Vertex(
						new Point3D(
							xmin + (double)i/xSteps * (xmax - xmin),
							ymin + (double)j/ySteps * (ymax - ymin),
							eval(grid[i, j].Position.X, grid[i, j].Position.Y)),
						new Point3D((double)i/xSteps, (double)j/ySteps, 0.0));
				}
			}

			for (int i = 0; i < xSteps; i++)
			{
				for (int j = 0; j < ySteps; j++)
				{
					Vector3D k0, k1, k2, k3;

					if (i != xSteps-1)
						k0 = (grid[i+1, j].Position - grid[i, j].Position).GetNormalized();
					else
						k0 = new Vector3D(1.0, 0.0, 0.0);

					if (j != 0)
						k1 = (grid[i, j-1].Position - grid[i, j].Position).GetNormalized();
					else
						k1 = new Vector3D(0.0, -1.0, 0.0);

					if (i != 0)
						k2 = (grid[i-1, j].Position - grid[i, j].Position).GetNormalized();
					else
						k2 = new Vector3D(-1.0, 0.0, 0.0);

					if (j != ySteps-1)
						k3 = (grid[i, j+1].Position - grid[i, j].Position).GetNormalized();
					else
						k3 = new Vector3D(0.0, 1.0, 0.0);

					grid[i, j].Normal = (
						(k1 % k0).GetNormalized() +
						(k2 % k1).GetNormalized() +
						(k3 % k2).GetNormalized() +
						(k0 % k3).GetNormalized()).GetNormalized();
				}
			}

			Mesh3D mesh = new Mesh3D();

			for (int i = 0; i < xSteps-1; i++)
			{
				for (int j = 0; j < ySteps-1; j++)
				{
					mesh.AddTriangle(grid[i+1, j], grid[i+1, j+1], grid[i, j]);
					mesh.AddTriangle(grid[i, j], grid[i+1, j+1], grid[i, j+1]);
				}
			}

			return mesh;
		}
	}
}
