using System;
using System.IO;

namespace Sharp3D
{
	public class MeshFactory
	{
		static public Mesh3D CreateCube(double size)
		{
			Mesh3D ret = new Mesh3D();

			double halfSize = 0.5 * size;

			ret.AddTriangle(
				new Vertex(new Point3D(-halfSize, -halfSize, -halfSize)),
				new Vertex(new Point3D(-halfSize, -halfSize,  halfSize)),
				new Vertex(new Point3D(-halfSize,  halfSize, -halfSize)));
			ret.AddTriangle(
				new Vertex(new Point3D(-halfSize,  halfSize, -halfSize)),
				new Vertex(new Point3D(-halfSize, -halfSize,  halfSize)),
				new Vertex(new Point3D(-halfSize,  halfSize,  halfSize)));

			ret.AddTriangle(
				new Vertex(new Point3D(-halfSize,  halfSize, -halfSize)),
				new Vertex(new Point3D(-halfSize,  halfSize,  halfSize)),
				new Vertex(new Point3D( halfSize,  halfSize, -halfSize)));
			ret.AddTriangle(
				new Vertex(new Point3D( halfSize,  halfSize, -halfSize)),
				new Vertex(new Point3D(-halfSize,  halfSize,  halfSize)),
				new Vertex(new Point3D( halfSize,  halfSize,  halfSize)));

			ret.AddTriangle(
				new Vertex(new Point3D( halfSize,  halfSize, -halfSize)),
				new Vertex(new Point3D( halfSize,  halfSize,  halfSize)),
				new Vertex(new Point3D( halfSize, -halfSize, -halfSize)));
			ret.AddTriangle(
				new Vertex(new Point3D( halfSize, -halfSize, -halfSize)),
				new Vertex(new Point3D( halfSize,  halfSize,  halfSize)),
				new Vertex(new Point3D( halfSize, -halfSize,  halfSize)));

			ret.AddTriangle(
				new Vertex(new Point3D( halfSize, -halfSize, -halfSize)),
				new Vertex(new Point3D( halfSize, -halfSize,  halfSize)),
				new Vertex(new Point3D(-halfSize, -halfSize, -halfSize)));
			ret.AddTriangle(
				new Vertex(new Point3D(-halfSize, -halfSize, -halfSize)),
				new Vertex(new Point3D( halfSize, -halfSize,  halfSize)),
				new Vertex(new Point3D(-halfSize, -halfSize,  halfSize)));

			ret.AddTriangle(
				new Vertex(new Point3D(-halfSize,  halfSize,  halfSize)),
				new Vertex(new Point3D(-halfSize, -halfSize,  halfSize)),
				new Vertex(new Point3D( halfSize,  halfSize,  halfSize)));
			ret.AddTriangle(
				new Vertex(new Point3D( halfSize,  halfSize,  halfSize)),
				new Vertex(new Point3D(-halfSize, -halfSize,  halfSize)),
				new Vertex(new Point3D( halfSize, -halfSize,  halfSize)));

			ret.AddTriangle(
				new Vertex(new Point3D(-halfSize, -halfSize, -halfSize)),
				new Vertex(new Point3D(-halfSize,  halfSize, -halfSize)),
				new Vertex(new Point3D( halfSize, -halfSize, -halfSize)));
			ret.AddTriangle(
				new Vertex(new Point3D( halfSize, -halfSize, -halfSize)),
				new Vertex(new Point3D(-halfSize,  halfSize, -halfSize)),
				new Vertex(new Point3D( halfSize,  halfSize, -halfSize)));

			return ret;
		}

		static public Mesh3D CreateCylinder(double height, double radius, int division)
		{
			Point2D[] circle = new Point2D[division];
			Vector3D[] sideNormals = new Vector3D[division];

			for (int i = 0; i < division; i++)
			{
				sideNormals[i] = new Vector3D(
					Math.Cos(2.0*Math.PI * i/division),
					Math.Sin(2.0*Math.PI * i/division),
					0.0);

				circle[i] = new Point2D(
					radius * sideNormals[i].X,
					radius * sideNormals[i].Y);
			}

			Mesh3D ret = new Mesh3D();

			for (int i = 0; i < division; i++)
			{
				// bottom disc segment
				ret.AddTriangle(
					new Vertex(new Point3D(circle[(i+1)%division].X, circle[(i+1)%division].Y, 0.0), new Vector3D(0.0, 0.0, -1.0)),
					new Vertex(new Point3D(circle[i].X, circle[i].Y, 0.0), new Vector3D(0.0, 0.0, -1.0)),
					new Vertex(new Point3D(0.0, 0.0, 0.0), new Vector3D(0.0, 0.0, -1.0)));

				// side
				ret.AddTriangle(
					new Vertex(new Point3D(circle[(i+1)%division].X, circle[(i+1)%division].Y, height), sideNormals[(i+1)%division]),
					new Vertex(new Point3D(circle[i].X, circle[i].Y, height), sideNormals[i]),
					new Vertex(new Point3D(circle[i].X, circle[i].Y, 0.0), sideNormals[i]));
				ret.AddTriangle(
					new Vertex(new Point3D(circle[i].X, circle[i].Y, 0.0), sideNormals[i]),
					new Vertex(new Point3D(circle[(i+1)%division].X, circle[(i+1)%division].Y, 0.0), sideNormals[(i+1)%division]),
					new Vertex(new Point3D(circle[(i+1)%division].X, circle[(i+1)%division].Y, height), sideNormals[(i+1)%division]));

				// top disc segment
				ret.AddTriangle(
					new Vertex(new Point3D(circle[i].X, circle[i].Y, height), new Vector3D(0.0, 0.0, 1.0)),
					new Vertex(new Point3D(circle[(i+1)%division].X, circle[(i+1)%division].Y, height), new Vector3D(0.0, 0.0, 1.0)),
					new Vertex(new Point3D(0.0, 0.0, height), new Vector3D(0.0, 0.0, 1.0)));
			}

			return ret;
		}

		static public Mesh3D CreateCylinder(Point3D start, Point3D end, double radius, int divisions,
			out Matrix transformation)
		{
			Vector3D axis = end - start;

			transformation = Matrix.CreateRotation(new Vector3D(0.0, 0.0, 1.0) % axis, Math.Acos(axis.GetNormalized().X));

			return CreateCylinder(axis.Length, radius, divisions);
		}

		static public Mesh3D CreateSphere(Point3D center, double radius, int divLat, int divLong)
		{
			Vertex[,] vertices = new Vertex[divLat+1, divLong+1];

			for (int i = 0; i <= divLat; i++)
			{
				// spherical coordinates

				double theta = Math.PI * i/divLat;
				double cosTheta = Math.Cos(theta), sinTheta = Math.Sin(theta);

				for (int k = 0; k <= divLong; k++)
				{
					double phi = 2.0*Math.PI * k/divLong;
					double cosPhi = Math.Cos(phi);
					double sinPhi = Math.Sin(phi);

					vertices[i, k] = new Vertex();

					vertices[i, k].Normal = new Vector3D(cosPhi*sinTheta, sinPhi*sinTheta, cosTheta);

					vertices[i, k].Position = new Point3D(
						radius * vertices[i, k].Normal.X,
						radius * vertices[i, k].Normal.Y,
						radius * vertices[i, k].Normal.Z);

					vertices[i, k].Tangent = new Vector3D(-sinPhi, cosPhi, 0.0);

					vertices[i, k].TextureCoordinate0.X = (double)k/divLong;
					vertices[i, k].TextureCoordinate0.Y = (double)i/divLat;
				}
			}

			Mesh3D ret = new Mesh3D();

			for (int i = 0; i < divLat; i++)
			{
				for (int k = 0; k < divLong; k++)
				{
					ret.AddTriangle(
						vertices[i, k],
						vertices[i+1, k],
						vertices[i+1, k+1]);

					ret.AddTriangle(
						vertices[i, k+1],
						vertices[i, k],
						vertices[i+1, k+1]);
				}
			}

			return ret;
		}

		static public Mesh3D CreateTorus(double outerRadius, double innerRadius, int divisions)
		{
			RotationMeshFactory factory = new RotationMeshFactory(divisions, true);

			for (int i = 0; i < divisions; i++)
				factory.AddPoint(outerRadius + innerRadius*Math.Cos(2.0*Math.PI * i/divisions), innerRadius*Math.Sin(2.0*Math.PI * i/divisions), true);

			return factory.CreateMesh();
		}

		static public Mesh3D CreateBottle(double size)
		{
			RotationMeshFactory factory = new RotationMeshFactory(32, false);

			factory.AddPoint(0.0, 0.0, false);
			factory.AddPoint(2.5 * size, 0.0, false);
			factory.AddPoint(2.5 * size, 11.0 * size, true);
			factory.AddPoint(2.4 * size, 12.0 * size, true);
			factory.AddPoint(2.0 * size, 13.0 * size, true);
			factory.AddPoint(1.0 * size, 14.0 * size, true);
			factory.AddPoint(1.0 * size, 20.0 * size, false);
			factory.AddPoint(0.0 * size, 20.0 * size, false);

			return factory.CreateMesh();
		}

		static public Mesh3D CreateAsteroid(string fileName)
		{
			Mesh3D mesh = new Mesh3D();

			FileStream stream = new FileStream(fileName, FileMode.Open, FileAccess.Read);
			BinaryReader reader = new BinaryReader(stream);

			int numVertices = reader.ReadInt32();
			int numIndices = reader.ReadInt32();

			Vertex[] vertices = new Vertex[numVertices];

			for (int i = 0; i < numVertices; i++)
			{
				vertices[i] = new Vertex();

				vertices[i].Position.X = (double)reader.ReadSingle();
				vertices[i].Position.Y = (double)reader.ReadSingle();
				vertices[i].Position.Z = (double)reader.ReadSingle();

				vertices[i].Normal.X = (double)reader.ReadSingle();
				vertices[i].Normal.Y = (double)reader.ReadSingle();
				vertices[i].Normal.Z = (double)reader.ReadSingle();
			}

			for (int i = 0; i < numIndices/3; i++)
			{
				int index0 = reader.ReadInt32();
				int index1 = reader.ReadInt32();
				int index2 = reader.ReadInt32();

				mesh.AddTriangle(vertices[index0], vertices[index1], vertices[index2]);
			}

			reader.Close();

			return mesh;
		}

		static public Mesh3D CreateQuadXZ(double width, double height)
		{
			Mesh3D mesh = new Mesh3D();

			Vertex vertex0 = new Vertex(
				new Point3D(-width, 0.0, -height),
				new Point3D(0.0, 0.0, 0.0));

			Vertex vertex1 = new Vertex(
				new Point3D(-width, 0.0, height),
				new Point3D(0.0, 1.0, 0.0));

			Vertex vertex2 = new Vertex(
				new Point3D(width, 0.0, height),
				new Point3D(1.0, 1.0, 0.0));

			Vertex vertex3 = new Vertex(
				new Point3D(width, 0.0, -height),
				new Point3D(1.0, 0.0, 0.0));

			mesh.AddTriangle(vertex1, vertex0, vertex2);
			mesh.AddTriangle(vertex2, vertex0, vertex3);

			return mesh;
		}

		static public Mesh3D CreateQuadXY(Rectangle2D rect)
		{
			Mesh3D mesh = new Mesh3D();

			Vertex vertex0 = new Vertex(
				new Point3D(rect.Left, -rect.Bottom, 0.0),
				new Point3D(0.0, 1.0, 0.0));

			Vertex vertex1 = new Vertex(
				new Point3D(rect.Left, -rect.Top, 0.0),
				new Point3D(0.0, 0.0, 0.0));

			Vertex vertex2 = new Vertex(
				new Point3D(rect.Right, -rect.Top, 0.0),
				new Point3D(1.0, 0.0, 0.0));

			Vertex vertex3 = new Vertex(
				new Point3D(rect.Right, -rect.Bottom, 0.0),
				new Point3D(1.0, 1.0, 0.0));

			mesh.AddTriangle(vertex1, vertex0, vertex2);
			mesh.AddTriangle(vertex2, vertex0, vertex3);

			return mesh;
		}
	}
}
