using System;

namespace Sharp3D
{
	public class MouseViewTransformer
	{
		private Point2D mouseLast;
		Camera camera;

		public MouseViewTransformer(Camera camera)
		{
			this.mouseLast = new Point2D();
			this.camera = camera;
		}

		public void MouseDown(int x, int y)
		{
			mouseLast.X = x;
			mouseLast.Y = y;
		}

		public void MoveInViewDirection(int x, int y)
		{
			double dy = 0.01*(y - mouseLast.Y);

			camera.Position -= dy * camera.Direction;
			camera.Center -= dy * camera.Direction;

			mouseLast.X = x;
			mouseLast.Y = y;
		}

		public void ZoomView(int x, int y)
		{
			double dy = y - mouseLast.Y;

			double lambda = Math.Pow(1.01, dy);

			Rectangle2D viewRect = camera.ViewRect;
			camera.ViewRect = new Rectangle2D(
				lambda * viewRect.Left,
				lambda * viewRect.Top,
				lambda * viewRect.Right,
				lambda * viewRect.Bottom);

			mouseLast.X = x;
			mouseLast.Y = y;
		}

		public void RotateView(int x, int y)
		{
			double dx = x - mouseLast.X;
			double dy = y - mouseLast.Y;

			Vector3D axis = camera.World2Camera.Inverted * new Vector3D(-dy, dx, 0.0);

			if (axis.Length > 0.001)
			{
				Matrix matrix = Matrix.CreateRotation(camera.Center, axis, 0.01*Math.Sqrt(dx*dx + dy*dy));

				camera.Position = matrix * camera.Position;
				camera.Up = matrix * camera.Up;
			}

			mouseLast.X = x;
			mouseLast.Y = y;
		}

		public void TranslateView(int x, int y)
		{
			double dx = 0.01*(x - mouseLast.X);
			double dy = 0.01*(y - mouseLast.Y);

			Vector3D translation = camera.World2Camera.Inverted * new Vector3D(-dx, -dy, 0.0);

			camera.Center += translation;
			camera.Position += translation;

			mouseLast.X = x;
			mouseLast.Y = y;
		}
	}
}
