using System;

namespace Sharp3D
{
	public class PositionalLight : ILight
	{
		private Color color;
		private Point3D position;
		private double attenuation;

		public PositionalLight(Color color, Point3D position, double attenuation)
		{
			this.color = color;
			this.position = position;
			this.attenuation = attenuation;
		}

		private double getGamma(double dist)
		{
			double t = attenuation * dist;
			return 1.0 / (t*t + 1.0);
		}

		public double CalcDiffuseWeight(Point3D position, Vector3D normal, Material material)
		{
			Vector3D direction = this.position - position;
			double dist = direction.Length;
			direction /= dist;

			double sc = direction * normal;

			if (sc < 0.0)
			{
				return 0.0;
			}
			else
			{
				return sc * getGamma(dist);
			}
		}

		public double CalcSpecularWeight(Point3D position, Vector3D normal, Point3D cameraPos, Material material)
		{
			Vector3D direction = this.position - position;
			double dist = direction.Length;
			direction /= dist;

			Vector3D reflect = 2.0*(direction*normal)*normal - direction;
			Vector3D view = (cameraPos - position).GetNormalized();

			double sc = view * reflect;

			if (sc < 0.0)
			{
				return 0.0;
			}
			else
			{
				return Math.Pow(sc, material.SpecularExponent) * getGamma(dist);
			}
		}

		public Point3D Position
		{
			get
			{
				return position;
			}
		}

		public ILight GetTransformed(Matrix m)
		{
			return new PositionalLight(color, m * position, attenuation);
		}

		public Color Color
		{
			get
			{
				return color;
			}
		}
	}
}
