using System;

namespace Sharp3D
{
	public class RenderingContext
	{
		private Surface surface;
		private Camera camera;
		private Matrix matrixLW;
		private Rectangle2D viewport;
		private bool backfaceCulling = true;
		private Color ambientColor;
		private ILight light;
		private IFog fog;
		private ushort[,] depthBuffer;

		public RenderingContext(Surface surface, Camera camera, Rectangle2D viewport, Color ambientColor, ILight light, IFog fog)
		{
			this.surface = surface;
			this.camera = camera;
			this.matrixLW = new Matrix();
			this.viewport = viewport;
			this.ambientColor = ambientColor;
			this.light = light;
			this.fog = fog;

			// 0 means maximum depth; this saves us from filling the buffer with 65535
			this.depthBuffer = new ushort[(int)viewport.Width, (int)viewport.Height];
		}

		public Surface Surface
		{
			get
			{
				return surface;
			}
		}

		public Camera Camera
		{
			get
			{
				return camera;
			}
		}

		public Rectangle2D Viewport
		{
			get
			{
				return viewport;
			}
		}

		public bool BackfaceCulling
		{
			get
			{
				return backfaceCulling;
			}

			set
			{
				backfaceCulling = value;
			}
		}

		public Matrix Local2World
		{
			get
			{
				return matrixLW;
			}

			set
			{
				matrixLW = value;
			}
		}

		public Matrix World2Camera
		{
			get
			{
				return camera.World2Camera;
			}
		}

		public Matrix Camera2Canonical
		{
			get
			{
				return camera.Camera2Canonical;
			}
		}

		public Matrix Canonical2Screen
		{
			get
			{
				return camera.Canonical2Screen;
			}
		}

		public Matrix Screen2Device
		{
			get
			{
				double w = viewport.Right - viewport.Left + 0.5;	// add bias to avoid fringes
				double h = viewport.Bottom - viewport.Top + 0.5;

				return new Matrix(
					0.5*w, 0.0, 0.0, 0.5*w + viewport.Left,
					0.0, 0.5*h, 0.0, 0.5*h + viewport.Top,
					0.0, 0.0, 65535.0, 0.0,
					0.0, 0.0, 0.0, 1.0);
			}
		}

		public Color AmbientColor
		{
			get
			{
				return ambientColor;
			}
		}

		public ILight Light
		{
			get
			{
				return light;
			}
		}

		public IFog Fog
		{
			get
			{
				return fog;
			}
		}

		public ushort[,] DepthBuffer
		{
			get
			{
				return depthBuffer;
			}
		}

		public double PerspectiveCorrection
		{
			get
			{
				return 65535 * camera.FarPlaneDistance / (camera.FarPlaneDistance - camera.NearPlaneDistance);
			}
		}
	}
}
