using System;
using System.Collections;

namespace Sharp3D
{
	public class RotationMeshFactory
	{
		private int steps;
		private bool closed;

		private class Point
		{
			public Point3D Position;
			public bool Smooth;

			public Point(double x, double y, bool smooth)
			{
				this.Position = new Point3D(x, y, 0.0);
				this.Smooth = smooth;
			}
		};

		private ArrayList points;

		private class GenTemplate
		{
			public Point3D Position;
			public Vector3D Normal;
			public double V;	// texture coordinate

			public GenTemplate(Point3D position, Vector3D normal, double V)
			{
				this.Position = position;
				this.Normal = normal;
				this.V = V;
			}
		};

		public RotationMeshFactory(int steps, bool closed)
		{
			this.steps = steps;
			this.closed = closed;

			this.points = new ArrayList();
		}

		public void AddPoint(double x, double y, bool smooth)
		{
			points.Add(new Point(x, y, smooth));
		}

		public Mesh3D CreateMesh()
		{
			double length = 0.0;

			Vector3D[] normals = new Vector3D[points.Count];
			double[] distance = new double[points.Count];

			for (int i = 0; i < points.Count; i++)
			{
				Vector3D diff = new Vector3D(((Point)points[(i+1)%points.Count]).Position.Y - ((Point)points[i]).Position.Y, ((Point)points[i]).Position.X - ((Point)points[(i+1)%points.Count]).Position.X, 0.0);

				normals[i] = diff.GetNormalized();

				distance[i] = diff.Length;
				length += distance[i];
			}

			ArrayList template = new ArrayList();

			if (closed)
			{
				if (((Point)points[0]).Smooth)
				{
					template.Add(new GenTemplate(((Point)points[0]).Position, new Vector3D(normals[normals.Length-1] + normals[0]).GetNormalized(), 0.0));
				}
				else
				{
					template.Add(new GenTemplate(((Point)points[0]).Position, normals[normals.Length-1], 0.0));
					template.Add(new GenTemplate(((Point)points[0]).Position, normals[0], 0.0));
				}

				double curLength = distance[0];

				for (int i = 1; i < points.Count; i++)
				{
					if (((Point)points[i]).Smooth)
					{
						template.Add(new GenTemplate(((Point)points[i]).Position, new Vector3D(normals[i-1] + normals[i]).GetNormalized(), curLength/length));
					}
					else
					{
						template.Add(new GenTemplate(((Point)points[i]).Position, normals[i-1], curLength/length));
						template.Add(new GenTemplate(((Point)points[i]).Position, normals[i], curLength/length));
					}

					curLength += distance[i];
				}
			}
			else
			{
				length -= distance[points.Count-1];

				template.Add(new GenTemplate(((Point)points[0]).Position, normals[0], 0.0));

				double curLength = distance[0];

				for (int i = 1; i < points.Count-1; i++)
				{
					if (((Point)points[i]).Smooth)
					{
						template.Add(new GenTemplate(((Point)points[i]).Position, new Vector3D(normals[i-1] + normals[i]).GetNormalized(), curLength/length));
					}
					else
					{
						template.Add(new GenTemplate(((Point)points[i]).Position, normals[i-1], curLength/length));
						template.Add(new GenTemplate(((Point)points[i]).Position, normals[i], curLength/length));
					}

					curLength += distance[i];
				}

				template.Add(new GenTemplate(((Point)points[points.Count-1]).Position, normals[normals.Length-2], curLength/length));
			}

			Vertex[,] vertices = new Vertex[steps, template.Count];

			for (int k = 0; k < steps; k++)
			{
				Matrix rotY = Matrix.CreateRotationY(-2.0*Math.PI * (double)k/steps);

				for (int i = 0; i < template.Count; i++)
				{
					vertices[k, i] = new Vertex(
						rotY * ((GenTemplate)template[i]).Position,
						rotY * ((GenTemplate)template[i]).Normal,
						new Point3D((double)k/steps, ((GenTemplate)template[i]).V, 0.0));
				}
			}

			Mesh3D mesh = new Mesh3D();

			for (int k = 0; k < steps; k++)
			{
				for (int i = 0; i < template.Count; i++)
				{
					mesh.AddTriangle(vertices[(k+1)%steps, i], vertices[(k+1)%steps, (i+1)%template.Count], vertices[k, i]);
					mesh.AddTriangle(vertices[k, i], vertices[(k+1)%steps, (i+1)%template.Count], vertices[k, (i+1)%template.Count]);
				}
			}

			return mesh;
		}
	}
}
