using System;

namespace Sharp3D
{
	public class Scene
	{
		public Camera Camera;
		public SceneGraphNode RootNode;
		public Color AmbientColor;
		public ILight Light;
		public IFog Fog;
		public ISceneBackground Background;

		public Scene()
		{
			Point3D position = new Point3D(1.0, 0.0, 0.0);
			Point3D center = new Point3D(0.0, 0.0, 0.0);
			Vector3D up = new Vector3D(0.0, 0.0, 1.0);

			Camera = new Camera(position, center, up, new Rectangle2D(-0.4, -0.3, 0.4, 0.3), 0.75, 8.0);

			AmbientColor = new Color(0, 0, 0);

			Light = new PositionalLight(new Color(255, 255, 255), new Point3D(0.0, 0.0, 0.0), 1.0);

			Fog = new NoFog();

			Background = new SolidBackground(new Color(255, 0, 0));
		}

		public void Render(Surface surface, Rectangle2D viewport)
		{
			RenderingContext context = new RenderingContext(surface, Camera, viewport, AmbientColor, Light, Fog);

			Background.RenderBackground(context);
			RootNode.Render(context);
		}
	}
}
