using System;
using System.Collections;

namespace Sharp3D
{
	public class SceneGraphNode
	{
		private ArrayList childNodes;
		private bool isVisible;
		private Matrix transformation;
		private Mesh3D mesh;
		private Material material;
		private Mesh3D.RenderingModes mode;

		public SceneGraphNode()
		{
			childNodes = new ArrayList();
			isVisible = true;
			transformation = new Matrix();
			mesh = new Mesh3D();

			material = new Material();
			material.AmbientCoefficient = 0.0;
			material.DiffuseCoefficient = 1.0;
			material.SpecularCoefficient = 0.0;
			material.SpecularExponent = 1.0;

			mode = Mesh3D.RenderingModes.WIREFRAME;
		}

		public bool IsVisible
		{
			get
			{
				return isVisible;
			}
			
			set
			{
				isVisible = value;
			}
		}

		public Matrix Transformation
		{
			get
			{
				return transformation;
			}
			
			set
			{
				transformation = value;
			}
		}

		public Material Material
		{
			get
			{
				return material;
			}

			set
			{
				material = value;
			}
		}

		public Mesh3D Mesh
		{
			get
			{
				return mesh;
			}
			
			set
			{
				mesh = value;
			}
		}

		public Mesh3D.RenderingModes RenderingMode
		{
			get
			{
				return mode;
			}

			set
			{
				mode = value;
			}
		}

		public SceneGraphNode GetChild(int i)
		{
			return (SceneGraphNode)childNodes[i];
		}

		public void AddChild(SceneGraphNode child)
		{
			childNodes.Add(child);
		}

		public void Render(RenderingContext context)
		{
			Matrix rootTransformation = context.Local2World;

			context.Local2World = rootTransformation * transformation;
			mesh.Render(context, material, mode);

			for (int i = 0; i < childNodes.Count; i++)
			{
				((SceneGraphNode)childNodes[i]).Render(context);
			}

			context.Local2World = rootTransformation;
		}
	}
}
