using System;

namespace Sharp3D
{
	public class SpaceBackground : ISceneBackground
	{
		private Vector3D[] stars;

		public SpaceBackground(int numStars)
		{
			stars = new Vector3D[numStars];

			Random rand = new Random(0);

			for (int i = 0; i < numStars; i++)
			{
				// spherical coordinates

				double phi = 2.0*Math.PI * rand.NextDouble();
				double theta = 2.0*Math.PI * rand.NextDouble();

				stars[i] = new Vector3D(
					Math.Cos(phi) * Math.Sin(theta),
					Math.Sin(phi) * Math.Sin(theta),
					Math.Cos(theta));
			}
		}

		public void RenderBackground(RenderingContext context)
		{
			Matrix matrixWC = context.World2Camera;
			Matrix matrixSD = context.Screen2Device;

			double x_min = context.Camera.ViewRect.Left / context.Camera.NearPlaneDistance;
			double x_max = context.Camera.ViewRect.Right / context.Camera.NearPlaneDistance;
			double y_min = context.Camera.ViewRect.Top / context.Camera.NearPlaneDistance;
			double y_max = context.Camera.ViewRect.Bottom / context.Camera.NearPlaneDistance;

			double w = 2.0 * context.Camera.NearPlaneDistance / (context.Camera.ViewRect.Right - context.Camera.ViewRect.Left);
			double h = 2.0 * context.Camera.NearPlaneDistance / (context.Camera.ViewRect.Bottom - context.Camera.ViewRect.Top);

			for (int i = 0; i < stars.Length; i++)
			{
				Vector3D dir = matrixWC * stars[i];

				Point3D p = new Point3D(-dir.Z / dir.X, dir.Y / dir.X, 1.0);

				if (x_min < p.X && p.X < x_max)
				{
					if (y_min < p.Y && p.Y < y_max)
					{
						p.X *= w;
						p.Y *= h;

						p = matrixSD * p;

						context.Surface.SetPixel((int)p.X, (int)p.Y, new Color(255, 255, 255));
					}
				}
			}
		}
	}
}
