using System;

namespace Sharp3D
{
	public class SpecularVertexShader : IVertexShader
	{
		private Matrix matrixLocal2Canonical;
		private Point3D posLight, posEye;	// local coordinate system

		public SpecularVertexShader(Matrix matrixLocal2Canonical, Point3D posLight, Point3D posEye)
		{
			this.matrixLocal2Canonical = matrixLocal2Canonical;
			this.posLight = posLight;
			this.posEye = posEye;
		}

		public Vertex calculate(Vertex input)
		{
			// transform position to clip space
			Vertex output = new Vertex(matrixLocal2Canonical * input.Position);

			// view, light and half vector
			Vector3D viewVec = (posEye - input.Position).GetNormalized();
			Vector3D lightVec = (posLight - input.Position).GetNormalized();
			Vector3D halfVec = (lightVec + viewVec).GetNormalized();

			// normal and half vector
			output.TextureCoordinate0 = new Point3D(input.Normal.X, input.Normal.Y, input.Normal.Z);
			output.TextureCoordinate1 = new Point3D(halfVec.X, halfVec.Y, halfVec.Z);

			return output;
		}
	}
}
