using System;

namespace Sharp3D
{
	public class Texture
	{
		private Image image;

		public enum TextureModes
		{
			NEAREST_NEIGHBOR,
			BILINEAR
		};

		private TextureModes mode;

		public Texture(Image image, TextureModes mode)
		{
			this.image = image;
			this.mode = mode;
		}

		public Color GetPixel(double u, double v)
		{
			if (u >= 1.0 || u < 0.0)
			{
				u -= Math.Floor(u);
			}

			if (v >= 1.0 || v < 0.0)
			{
				v -= Math.Floor(v);
			}

			double x = u*image.Width;
			double y = v*image.Height;

			if (mode == TextureModes.NEAREST_NEIGHBOR)
			{
				return image[(int)x, (int)y];
			}
			else
			{
				int x_tex = (int)x;
				int y_tex = (int)y;

				double alpha = x - x_tex;
				double beta = y - y_tex;

				return
					(1.0 - alpha)*(1.0 - beta)*image[x_tex, y_tex] +
					(1.0 - alpha)*beta*image[x_tex, (y_tex + 1)%image.Height] +
					alpha*(1.0 - beta)*image[(x_tex + 1)%image.Width, y_tex] +
					alpha*beta*image[(x_tex + 1)%image.Width, (y_tex + 1)%image.Height];
			}
		}
	}
}
