using System;

namespace Sharp3D
{
	public class Vertex
	{
		public Point3D Position;
		public Vector3D Normal;
		public Vector3D Tangent;
		public double DiffuseWeight;
		public double SpecularWeight;
		public Point3D TextureCoordinate0;
		public Point3D TextureCoordinate1;
		public Point3D TextureCoordinate2;

		public Vertex()
		{
			this.Position = new Point3D();
			this.Normal = new Vector3D(0.0, 0.0, 1.0);
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.TextureCoordinate0 = new Point3D();
			this.TextureCoordinate1 = new Point3D();
			this.TextureCoordinate2 = new Point3D();
		}

		public Vertex(Point3D position)
		{
			this.Position = position;
			this.Normal = new Vector3D(0.0, 0.0, 1.0);
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.TextureCoordinate0 = new Point3D();
			this.TextureCoordinate1 = new Point3D();
			this.TextureCoordinate2 = new Point3D();
		}

		public Vertex(Point3D position, Vector3D normal)
		{
			this.Position = position;
			this.Normal = normal;
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.TextureCoordinate0 = new Point3D();
			this.TextureCoordinate1 = new Point3D();
			this.TextureCoordinate2 = new Point3D();
		}

		public Vertex(Point3D position, Point3D textureCoordinate0)
		{
			this.Position = position;
			this.Normal = new Vector3D(0.0, 0.0, 1.0);
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.TextureCoordinate0 = textureCoordinate0;
			this.TextureCoordinate1 = new Point3D();
			this.TextureCoordinate2 = new Point3D();
		}

		public Vertex(Point3D position, Vector3D normal, Point3D textureCoordinate0)
		{
			this.Position = position;
			this.Normal = normal;
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.TextureCoordinate0 = textureCoordinate0;
			this.TextureCoordinate1 = new Point3D();
			this.TextureCoordinate2 = new Point3D();
		}

		public Vertex(Point3D textureCoordinate0, Point3D textureCoordinate1, Point3D textureCoordinate2)
		{
			this.Position = new Point3D();
			this.Normal = new Vector3D(0.0, 0.0, 1.0);
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.TextureCoordinate0 = textureCoordinate0;
			this.TextureCoordinate1 = textureCoordinate1;
			this.TextureCoordinate2 = textureCoordinate2;
		}

		public Vertex(Point3D position, Vector3D normal, double diffuseWeight, double specularWeight)
		{
			this.Position = position;
			this.Normal = normal;
			this.Tangent = new Vector3D(1.0, 0.0, 0.0);
			this.DiffuseWeight = diffuseWeight;
			this.SpecularWeight = specularWeight;
			this.TextureCoordinate0 = new Point3D();
			this.TextureCoordinate1 = new Point3D();
			this.TextureCoordinate2 = new Point3D();
		}

		public Vertex(Point3D position, Vector3D normal, Vector3D tangent,
			double diffuseWeight, double specularWeight,
			Point3D textureCoordinate0, Point3D textureCoordinate1, Point3D textureCoordinate2)
		{
			this.Position = position;
			this.Normal = normal;
			this.Tangent = tangent;
			this.DiffuseWeight = diffuseWeight;
			this.SpecularWeight = specularWeight;
			this.TextureCoordinate0 = textureCoordinate0;
			this.TextureCoordinate1 = textureCoordinate1;
			this.TextureCoordinate2 = textureCoordinate2;
		}
	}
}
