private Texture diffuseTexture;
private Texture normalTexture;
private Texture horizonTexture0, horizonTexture1;
private Texture trigonometryTexture0, trigonometryTexture1;

public EarthPixelShader(Texture diffuseTexture, Texture normalTexture,
Texture horizonTexture0, Texture horizonTexture1,
Texture trigonometryTexture0, Texture trigonometryTexture1)
{
this.diffuseTexture = diffuseTexture;
this.normalTexture = normalTexture;
this.horizonTexture0 = horizonTexture0;
this.horizonTexture1 = horizonTexture1;
this.trigonometryTexture0 = trigonometryTexture0;
this.trigonometryTexture1 = trigonometryTexture1;
}


public Color calculate(Material material, Vector3D t0, Vector3D t1, Vector3D t2)
{
			// diffuse color

Color diffuseColor = diffuseTexture.GetPixel(t0.X, t0.Y);


			// light and half vector

Vector3D lightVec = t1.GetNormalized();
Vector3D halfVec = t2.GetNormalized();


			// diffuse and specular lighting

Color c = normalTexture.GetPixel(t0.X, t0.Y);
Vector3D normal = new Vector3D(c.RedRange, c.GreenRange, c.BlueRange);

Color outputColor =
Math.Max(normal*lightVec, 0.0)*diffuseColor +
Math.Pow(Math.Max(normal*halfVec, 0.0), 18.0)*new Color(255, 255, 255);


			// self-shadowing

			// elevations from horizon maps
Color elev0 = horizonTexture0.GetPixel(t0.X, t0.Y);
Color elev1 = horizonTexture1.GetPixel(t0.X, t0.Y);

			// angle dependent weights from trigonometry textures
			// use (lightVec.x scaled biased, lightVec.y scaled biased) as coordinates
float coordTrigU = 0.5*lightVec.X + 0.5;
float coordTrigV = 0.5*lightVec.Y + 0.5;

Color weight0 = trigonometryTexture0.GetPixel(coordTrigU);
Color weight1 = trigonometryTexture1.GetPixel(coordTrigV);

			// elevation values scaled by weights
float elev = (elev0*weight0 + elev1*weight1) / 255.0;

			// elevation difference between surface and light vector scaled by a constant factor
float shadow = 2.0 * (lightVec.Z - elev);


return shadow * outputColor;
}


Texture texture = new Texture(new Image("testQ8W8V8U8.png"), Texture.TextureModes.NEAREST_NEIGHBOR);
Color color = texture.GetPixel(0, 0);
int q = color.Q8;
int w = color.W8;


Color outputColor = new Color((int)(Math.Max(255.0*lightVec.X, 0.0)), (int)(Math.Max(255.0*lightVec.Y, 0.0)), (int)(Math.Max(255.0*lightVec.Z, 0.0)));

return new Color((int)(255.0 * (0.5*elev0.U8Scaled+0.5)), (int)(255.0 * (0.5*elev0.V8Scaled+0.5)), (int)(255.0 * (0.5*elev0.W8Scaled+0.5)));

double elev =
elev0.U8Scaled * weight0.AlphaScaled + elev0.V8Scaled * weight0.RedScaled + elev0.W8Scaled * weight0.GreenScaled + elev0.Q8Scaled * weight0.BlueScaled +
elev1.U8Scaled * weight1.AlphaScaled + elev1.V8Scaled * weight1.RedScaled + elev1.W8Scaled * weight1.GreenScaled + elev1.Q8Scaled * weight1.BlueScaled;
